@@ -23,9 +23,9 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "wpe-fdo/initialize-egl.h"
+#include "wpe/initialize-egl.h"
 
-#include "ws.h"
+#include "ws-egl.h"
 
 extern "C" {
 
@@ -33,7 +33,11 @@ __attribute__((visibility("default")))
 bool
 wpe_fdo_initialize_for_egl_display(EGLDisplay display)
 {
-    return WS::Instance::singleton().initialize(display);
+    if (!WS::Instance::isConstructed())
+        WS::Instance::construct(std::unique_ptr<WS::ImplEGL>(new WS::ImplEGL));
+
+    auto& instance = WS::Instance::singleton();
+    return static_cast<WS::ImplEGL&>(instance.impl()).initialize(display);
 }
 
 }
