@@ -24,6 +24,7 @@
  */
 
 #include "exported-buffer-shm-private.h"
+#include "linux-dmabuf/linux-dmabuf.h"
 #include "view-backend-exportable-private.h"
 #include "ws.h"
 #include <cassert>
@@ -114,6 +115,11 @@ public:
         client->export_shm_buffer(data, buffer);
     }
 
+    void exportEGLStreamProducer(struct wl_resource* bufferResource) override
+    {
+        assert(!"should not be reached");
+    }
+
     void releaseBuffer(struct wl_resource* buffer)
     {
         BufferResource* matchingResource = nullptr;
